#ifndef NDPluginFFT_H
#define NDPluginFFT_H

#include <epicsTypes.h>
#include <asynStandardInterfaces.h>

#include "NDPluginDriver.h"

/** Map parameter enums to strings that will be used to set up EPICS databases
  */
#define NDPluginFFTScaleFactorString "FFT_SCALE_FACTOR" /* (asyncFloat64, r/w) Scale factor */
#define NDPluginFFTDataTypeString    "FFT_DATA_TYPE"    /* (asynInt32,    r/w) Output type.  -1 means automatic. */
#define NDPluginFFTDirectionString   "FFT_DIRECTION"    /* (asynInt32,    r/w) FFT Direction */

static const char* pluginName = "NDPluginFFT";

/** Perform FFTs on NDArrays.   */
class epicsShareClass NDPluginFFT : public NDPluginDriver {
public:
    NDPluginFFT(const char *portName, int queueSize, int blockingCallbacks,
                 const char *NDArrayPort, int NDArrayAddr,
                 int maxBuffers, size_t maxMemory,
                 int priority, int stackSize);
    /* These methods override the virtual methods in the base class */
    void processCallbacks(NDArray *pArray);

protected:
    int P_ScaleFactor_;
    #define FIRST_FFT_PARAM P_ScaleFactor_
    int P_DataType_;
    int P_Direction_;
    #define LAST_FFT_PARAM P_Direction_

private:
    template <typename epicsType> asynStatus computeFFT(NDArray *pArray);

};
#define NUM_FFT_PARAMS ((int)(&LAST_FFT_PARAM - &FIRST_FFT_PARAM + 1))

#endif
