/*
 * NDPluginFFT.cpp
 *
 * FFT plugin
 * Author: Mark Rivers
 *
 * Created January 29, 2015
 *
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <math.h>

#include <iocsh.h>

#include <asynDriver.h>

#include <epicsExport.h>
#include "NDPluginDriver.h"
#include "NDPluginFFT.h"

extern "C" {
void fourn(float data[], unsigned long nn[], int ndim, int isign);
}

static const char *driverName = "NDPluginFFT";

template <typename epicsType>
asynStatus NDPluginFFT::computeFFT(NDArray *pArray)
{
  unsigned long ldims[2];
  size_t dims[2];
  size_t sizeX, sizeY;
  size_t i, j;
  float *FFT;
  int FFTDirection;
  double scaleFactor;
  int outputDataType;
  NDArray *pTemp, *pArrayOut;
  epicsType *pDataIn = (epicsType *)pArray->pData;
  epicsType *pIn;
  float *pDataOut;
  float *pOut;
  NDArrayInfo arrayInfo;

  // Get information about the input array
  
  
  // Get the current values of the FFTDirection, scaleFactor, and outputDataType parameters

  /* Release the lock; The code after this is computationally intensive and does not access any shared data */

  // The fourn function only works on dimensions that are powers of 2 so find the first power of
  // 2 that is >= input dimension

  // Create float array which has 2 * sizeX * sizeY, because FFT is done in place and output is complex
  
  // Copy the data from the input array to the FFT float array.  Skip every other element in output

  // Take the FFT of the array

  
  // Allocate an NDArray of type NDFloat32
  
  // Copy the data from FFT array to the output array.  
  // Take abs of each complex number and multiply by scale factor

  
  // Convert the output array to the desired output data type

  // Free the FFT array and the temporary NDArray
  
  // Copy the uniqueId, timestamps and attribute list from the input array to the output array.
  
  // Take the lock again, we are done with the computationally intensive task

}

/** Callback function that is called by the NDArray driver with new NDArray data.
  * Grabs the current NDArray and applies the selected FFTs to the data.  Apply the FFTs in order.
  * \param[in] pArray  The NDArray from the callback.
  */
void NDPluginFFT::processCallbacks(NDArray *pArray){
  
  asynStatus status;
  static const char* functionName = "processCallbacks";

  /* Call the base class method */


  // Make sure input array is 2-D

  /* Previous version of the array was held in memory.  Release it. */

  // Call template function with the appropriate data type
  
  // Do the generic pointer callbacks on the new array
  
}


/** Constructor for NDPluginFFT; most parameters are simply passed to NDPluginDriver::NDPluginDriver.
  * After calling the base class constructor this method sets reasonable default values for all of the
  * FFT parameters.
  * \param[in] portName The name of the asyn port driver to be created.
  * \param[in] queueSize The number of NDArrays that the input queue for this plugin can hold when
  *      NDPluginDriverBlockingCallbacks=0.  Larger queues can decrease the number of dropped arrays,
  *      at the expense of more NDArray buffers being allocated from the underlying driver's NDArrayPool.
  * \param[in] blockingCallbacks Initial setting for the NDPluginDriverBlockingCallbacks flag.
  *      0=callbacks are queued and executed by the callback thread; 1 callbacks execute in the thread
  *      of the driver doing the callbacks.
  * \param[in] NDArrayPort Name of asyn port driver for initial source of NDArray callbacks.
  * \param[in] NDArrayAddr asyn port driver address for initial source of NDArray callbacks.
  * \param[in] maxBuffers The maximum number of NDArray buffers that the NDArrayPool for this driver is
  *      allowed to allocate. Set this to -1 to allow an unlimited number of buffers.
  * \param[in] maxMemory The maximum amount of memory that the NDArrayPool for this driver is
  *      allowed to allocate. Set this to -1 to allow an unlimited amount of memory.
  * \param[in] priority The thread priority for the asyn port driver thread if ASYN_CANBLOCK is set in asynFlags.
  * \param[in] stackSize The stack size for the asyn port driver thread if ASYN_CANBLOCK is set in asynFlags.
  */
NDPluginFFT::NDPluginFFT(const char *portName, int queueSize, int blockingCallbacks,
             const char *NDArrayPort, int NDArrayAddr, int maxBuffers, size_t maxMemory,
             int priority, int stackSize)
  /* Invoke the base class constructor */
  : NDPluginDriver(portName, queueSize, blockingCallbacks,
                   NDArrayPort, NDArrayAddr, 1, NUM_FFT_PARAMS, maxBuffers, maxMemory,
                   asynInt32ArrayMask | asynFloat64ArrayMask | asynGenericPointerMask,
                   asynInt32ArrayMask | asynFloat64ArrayMask | asynGenericPointerMask,
                   ASYN_MULTIDEVICE, 1, priority, stackSize)
{
  //static const char *functionName = "NDPluginFFT";

  // Create the new parameters

  /* Set the plugin type string */

  /* Try to connect to the array port */
}

/** Configuration command */
extern "C" int NDFFTConfigure(const char *portName, int queueSize, int blockingCallbacks,
                                    const char *NDArrayPort, int NDArrayAddr,
                                    int maxBuffers, size_t maxMemory,
                                    int priority, int stackSize)
{
  new NDPluginFFT(portName, queueSize, blockingCallbacks, NDArrayPort, NDArrayAddr,
              maxBuffers, maxMemory, priority, stackSize);
  return(asynSuccess);
}

/* EPICS iocsh shell commands */
static const iocshArg initArg0 = { "portName",iocshArgString};
static const iocshArg initArg1 = { "frame queue size",iocshArgInt};
static const iocshArg initArg2 = { "blocking callbacks",iocshArgInt};
static const iocshArg initArg3 = { "NDArrayPort",iocshArgString};
static const iocshArg initArg4 = { "NDArrayAddr",iocshArgInt};
static const iocshArg initArg5 = { "maxBuffers",iocshArgInt};
static const iocshArg initArg6 = { "maxMemory",iocshArgInt};
static const iocshArg initArg7 = { "priority",iocshArgInt};
static const iocshArg initArg8 = { "stackSize",iocshArgInt};
static const iocshArg * const initArgs[] = {&initArg0,
                                            &initArg1,
                                            &initArg2,
                                            &initArg3,
                                            &initArg4,
                                            &initArg5,
                                            &initArg6,
                                            &initArg7,
                                            &initArg8};
static const iocshFuncDef initFuncDef = {"NDFFTConfigure",9,initArgs};
static void initCallFunc(const iocshArgBuf *args)
{
  NDFFTConfigure(args[0].sval, args[1].ival, args[2].ival,
                       args[3].sval, args[4].ival, args[5].ival,
                       args[6].ival, args[7].ival, args[8].ival);
}

extern "C" void NDFFTRegister(void)
{
  iocshRegister(&initFuncDef,initCallFunc);
}

extern "C" {
epicsExportRegistrar(NDFFTRegister);
}
